use maud::{Markup, Render};

use crate::auth::User;

use crate::ui::basic::*;

use super::UIWidget;

#[allow(non_snake_case)]
pub fn AppBar(name: &str, user: Option<User>) -> AppBarWidget {
    AppBarWidget {
        name: name.to_owned(),
        user,
    }
}

pub struct AppBarWidget {
    name: String,
    user: Option<User>,
}

impl Render for AppBarWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for AppBarWidget {
    fn can_inherit(&self) -> bool {
        false
    }

    fn render_with_class(&self, _: &str) -> Markup {
        Padding(Shadow::medium(Background(
            Gray::_800,
            Header(
                Padding(
                    Flex(
                        Div()
                            .vanish()
                            .add(
                                Flex(Link(
                                    "/",
                                    Div()
                                        .vanish()
                                        .add(Sized(
                                            10,
                                            10,
                                            RoundedMedium(Image("/favicon").alt("Logo")),
                                        ))
                                        .add(Span(&self.name).semibold().xl().white()),
                                ))
                                .items_center()
                                .space_x(2),
                            )
                            .add_some(self.user.as_ref(), |user| Text(&user.username).white()),
                    )
                    .group()
                    .justify(Justify::Between)
                    .items_center(),
                )
                .x(6),
            ),
        )))
        .y(2)
        .render()
    }
}
