use maud::{Markup, Render, html};

use super::UIWidget;

pub struct Aspect {
    kind: u8,
    inner: Box<dyn UIWidget>,
}

impl Aspect {
    pub fn auto<T: UIWidget + 'static>(inner: T) -> Self {
        Self {
            kind: 0,
            inner: Box::new(inner),
        }
    }

    pub fn square<T: UIWidget + 'static>(inner: T) -> Self {
        Self {
            kind: 1,
            inner: Box::new(inner),
        }
    }

    pub fn video<T: UIWidget + 'static>(inner: T) -> Self {
        Self {
            kind: 2,
            inner: Box::new(inner),
        }
    }
}

impl Render for Aspect {
    fn render(&self) -> Markup {
        let class = match self.kind {
            0 => "aspect-auto",
            1 => "aspect-square",
            2 => "aspect-video",
            _ => "",
        };

        if self.inner.as_ref().can_inherit() {
            html! {
                (self.inner.as_ref().render_with_class(class))
            }
        } else {
            html! {
                div class=(class) {
                    (self.inner.as_ref())
                }
            }
        }
    }
}
